@@ -7,9 +7,24 @@ classes and methods provided here.
 
 import gpod
 import types
-import eyeD3
+from mutagen.mp3 import MP3
+import mutagen.id3
 import gtkpod
 import os
+import locale
+import socket
+import datetime
+
+if hasattr(gpod, 'HAVE_GDKPIXBUF') and hasattr(gpod, 'HAVE_PYGOBJECT'):
+    try:
+        import gtk
+        pixbuf_support = True
+    except ImportError:
+        pixbuf_support = False
+else:
+    pixbuf_support = False
+
+defaultencoding = locale.getpreferredencoding()
 
 class DatabaseException(RuntimeError):
     """Exception for database errors."""
@@ -19,6 +34,10 @@ class TrackException(RuntimeError):
     """Exception for track errors."""
     pass
 
+class PhotoException(RuntimeError):
+    """Exception for track errors."""
+    pass
+
 class Database:
     """An iTunes database.
 
@@ -92,7 +111,7 @@ class Database:
 
         if not gpod.itdb_write_file(self._itdb, self._itdb_file, None):
             raise DatabaseException("Unable to save iTunes database %s" % self)
-        itdbext_file = "%s.ext" % (self._itdb_file)        
+        itdbext_file = "%s.ext" % (self._itdb_file)
         gtkpod.write(itdbext_file, self, self._itdb_file)
 
     def __getitem__(self, index):
@@ -159,7 +178,7 @@ class Database:
                     pl.remove(item)
             if harddisk:
                 try:
-                    filename = item['userdata']['filename_locale']
+                    filename = item._userdata_into_default_locale('filename')
                 except KeyError, e:
                     raise TrackException("Unable to remove %s from hard disk, no filename available." % item)
                 os.unlink(filename)
@@ -186,7 +205,7 @@ class Database:
 
     Master   = property(get_master)
     Podcasts = property(get_podcasts)
-    Playlists= property(get_playlists) 
+    Playlists= property(get_playlists)
 
     def smart_update(self):
         """Update all smart playlists."""
@@ -214,7 +233,7 @@ class Database:
             self.Podcasts.add(track)
         else:
             self.Master.add(track)
-        track.__database = self # so the db doesn't get gc'd            
+        track.__database = self # so the db doesn't get gc'd
         return track
 
     def copy_delayed_files(self,callback=False):
@@ -238,7 +257,7 @@ class Database:
         for track in self:
             try:
                 transferred = int(track['userdata']['transferred'])
-            except KeyError:
+            except (KeyError, TypeError):
                 transferred = 1
             if not transferred:
                 to_copy.append(track)
@@ -267,23 +286,12 @@ class Track:
     # Note we don't free the underlying structure, as it's still used
     # by the itdb.
 
-    _proxied_attributes = ("title","ipod_path","album","artist","genre","filetype",
-                           "comment","category","composer","grouping","description",
-                           "podcasturl","podcastrss","chapterdata","subtitle","id",
-                           "size","tracklen","cd_nr","cds","track_nr","tracks",
-                           "bitrate","samplerate","samplerate_low","year","volume",
-                           "soundcheck","time_added","time_played","time_modified",
-                           "bookmark_time","rating","playcount","playcount2",
-                           "recent_playcount","transferred","BPM","app_rating",
-                           "type1","type2","compilation","starttime","stoptime",
-                           "checked","dbid","drm_userid","visible","filetype_marker",
-                           "artwork_count","artwork_size","samplerate2", "remember_playback_position",
-                           "time_released","has_artwork","flag4", "skip_when_shuffling",
-                           "lyrics_flag","movie_flag","mark_unplayed","samplecount",
-                           "chapterdata_raw","chapterdata_raw_length","artwork",
-                           "usertype")
+    _proxied_attributes = [k for k in gpod._Itdb_Track.__dict__.keys()
+                            if not (k.startswith("_") or
+                                    k.startswith("reserved") or
+                                    k == "chapterdata")]
 
-    def __init__(self, filename=None, from_file=None,
+    def __init__(self, filename=None,
                  proxied_track=None, podcast=False, ownerdb=None):
         """Create a Track object.
 
@@ -294,45 +302,48 @@ class Track:
         object.
 
         If podcast is True then the track will be setup as a Podcast,
-        unless proxied_track is set. 
+        unless proxied_track is set.
 
         """
 
-        # XXX couldn't from_file and filename be merged? 
-        if from_file:
-            filename = from_file
         if filename:
             self._track = gpod.itdb_track_new()
-            self['userdata'] = {'filename_locale': filename,
-                                'transferred': 0}
+            self['userdata'] = {'transferred': 0,
+                                'hostname': socket.gethostname(),
+                                'charset':defaultencoding}
+            self['userdata']['pc_mtime'] = os.stat(filename).st_mtime
+            self._set_userdata_utf8('filename',filename)
+            possible_image = os.path.join(os.path.split(filename)[0],'folder.jpg')
+            if os.path.exists(possible_image):
+                self.set_thumbnail(possible_image)
             try:
-                audiofile = eyeD3.Mp3AudioFile(self['userdata']['filename_locale'])
-            except eyeD3.tag.InvalidAudioFormatException, e:
+                audiofile = MP3(self._userdata_into_default_locale('filename'))
+            except Exception, e:
                 raise TrackException(str(e))
-            tag = audiofile.getTag()
-            for func, attrib in (('getArtist','artist'),
-                                 ('getTitle','title'),
-                                 ('getBPM','BPM'),
-                                 ('getPlayCount','playcount'),
-                                 ('getAlbum','album')):
-                value = getattr(tag,func)()
-                if value:
-                    self[attrib] = value
-            try:
-                self['genre'] = tag.getGenre().name
-            except AttributeError:
-                pass
-            disc, of = tag.getDiscNum()
-            if disc is not None:
-                self['cd_nr'] = disc
-            if of is not None:
-                self['cds'] = of
-            n, of = tag.getTrackNum()
-            if n is not None:
-                self['track_nr'] = n
-            if of is not None:
-                self['tracks'] = of
-            self['tracklen'] = audiofile.getPlayTime() * 1000
+            for tag, attrib in (('TPE1','artist'),
+                                ('TIT2','title'),
+                                ('TBPM','BPM'),
+                                ('TCON','genre'),
+                                ('TALB','album'),
+                                ('TPOS',('cd_nr','cds')),
+                                ('TRCK',('track_nr','tracks'))):
+                try:
+                    value = audiofile[tag]
+                    if isinstance(value,mutagen.id3.NumericPartTextFrame):
+                        parts = map(int,value.text[0].split("/"))
+                        if len(parts) == 2:
+                            self[attrib[0]], self[attrib[1]] = parts
+                        elif len(parts) == 1:
+                            self[attrib[0]] = parts[0]
+                    elif isinstance(value,mutagen.id3.TextFrame):
+                        self[attrib] = value.text[0].encode('UTF-8','replace')
+                except KeyError:
+                    pass
+            if self['title'] is None:
+                self['title'] = os.path.splitext(
+                    os.path.split(filename)[1])[0].decode(
+                    defaultencoding).encode('UTF-8')
+            self['tracklen'] = int(audiofile.info.length * 1000)
             self.set_podcast(podcast)
         elif proxied_track:
             self._track = proxied_track
@@ -341,18 +352,62 @@ class Track:
             self._track = gpod.itdb_track_new()
             self.set_podcast(podcast)
 
+    def _set_userdata_utf8(self, key, value):
+        self['userdata']['%s_locale' % key] = value
+        try:
+            self['userdata']['%s_utf8'   % key] = value.decode(self['userdata']['charset']).encode('UTF-8')
+        except UnicodeDecodeError, e:
+            # string clearly isn't advertised charset.  I prefer to
+            # not add the _utf8 version as we can't actually generate
+            # it. Maybe we'll have to populate a close-fit though.
+            pass
+
+    def _userdata_into_default_locale(self, key):
+        # to cope with broken filenames, we should trust the _locale version more,
+        # an even that may not really be in self['userdata']['charset']
+        if self['userdata']['charset'] == defaultencoding:
+            # don't try translate it or check it's actually valid, in case it isn't.
+            return self['userdata']['%s_locale' % key]
+        # our filesystem is in a different encoding to the filename, so
+        # try to convert it. The UTF-8 version is likely best to try first?
+        if self['userdata'].has_key('%s_utf8' % key):
+            unicode_value = self['userdata']['%s_utf8' % key].decode('UTF-8')
+        else:
+            unicode_value = self['userdata']['%s_locale' % key].decode(self['userdata']['charset'])
+        return unicode_value.encode(defaultencoding)
+
+    def set_coverart_from_file(self, filename):
+        gpod.itdb_track_set_thumbnails(self._track, filename)
+        self._set_userdata_utf8('thumbnail', filename)
+
+    def set_coverart(self, pixbuf):
+        if pixbuf == None:
+            gpod.itdb_track_remove_thumbnails(self._track)
+        elif isinstance(pixbuf, Photo):
+            raise NotImplemented("Can't set coverart from existing coverart yet")
+        else:
+            gpod.itdb_track_set_thumbnails_from_pixbuf(self._track,
+                                                       pixbuf)
+
+    def get_coverart(self):
+        return Photo(proxied_photo=self._track.artwork,
+                     ownerdb=self._track.itdb)
+
     def copy_to_ipod(self):
         """Copy the track to the iPod."""
-        self['userdata']['md5_hash'] = gtkpod.sha1_hash(
-            self['userdata']['filename_locale'])
-        if not gpod.itdb_get_mountpoint(self._track.itdb):
+        self['userdata']['sha1_hash'] = gtkpod.sha1_hash(self._userdata_into_default_locale('filename'))
+        mp = gpod.itdb_get_mountpoint(self._track.itdb)
+        if not mp:
             return False
-        self['userdata']['transferred'] = 1
         if gpod.itdb_cp_track_to_ipod(self._track,
-                                      self['userdata']['filename_locale'], None) != 1:
+                                      self._userdata_into_default_locale('filename'),
+                                      None) != 1:
             raise TrackException('Unable to copy %s to iPod as %s' % (
-                self['userdata']['filename_locale'],
+                self._userdata_into_default_locale('filename'),
                 self))
+        fname = self.ipod_filename().replace(mp, '').replace(os.path.sep, ':')
+        self['userdata']['filename_ipod'] = fname
+        self['userdata']['transferred'] = 1
         return True
 
     def ipod_filename(self):
@@ -400,31 +455,31 @@ class Track:
         return [self[k] for k in self._proxied_attributes]
 
     def pairs(self):
-        return [(k, self[k]) for k in self._proxied_attributes]        
+        return [(k, self[k]) for k in self._proxied_attributes]
 
     def __getitem__(self, item):
         if item == "userdata":
             return gpod.sw_get_track_userdata(self._track)
         elif item in self._proxied_attributes:
-            return getattr(self._track, item)
+            if item.startswith("time_"):
+                return datetime.datetime.fromtimestamp(getattr(self._track, item))
+            else:
+                return getattr(self._track, item)
         else:
             raise KeyError('No such key: %s' % item)
 
     def __setitem__(self, item, value):
-        #print item, value
         if item == "userdata":
             gpod.sw_set_track_userdata(self._track, value)
             return
         if type(value) == types.UnicodeType:
-            value = value.encode()
+            value = value.encode('UTF-8','replace')
         if item in self._proxied_attributes:
             return setattr(self._track, item, value)
         else:
             raise KeyError('No such key: %s' % item)
 
-# XXX would this be better as a public variable so that the docs would
-# list valid sort order values?
-_playlist_sorting = {
+playlist_sorting = {
     1:'playlist',
     2:'unknown2',
     3:'songtitle',
@@ -608,14 +663,12 @@ class Playlist:
 
     def get_sort(self):
         """Get the sort order for the playlist."""
-        return _playlist_sorting[self._pl.sortorder]
+        return playlist_sorting[self._pl.sortorder]
 
-    # XXX how does one find out what values are allowed for order without
-    # poking into this file?  (See similar question @ _playlist_order)
     def set_sort(self, order):
         """Set the sort order for the playlist."""
         order = order.lower()
-        for k, v in _playlist_sorting.items():
+        for k, v in playlist_sorting.items():
             if v == order:
                 self._pl.sortorder = v
                 return
@@ -625,7 +678,7 @@ class Playlist:
     id     = property(get_id)
     smart  = property(get_smart)
     master = property(get_master)
-    podcast= property(get_podcast)    
+    podcast= property(get_podcast)
     order  = property(get_sort, set_sort)
 
     def __str__(self):
@@ -685,3 +738,292 @@ class Playlist:
             gpod.itdb_playlist_remove_track(self._pl, track._track)
         else:
             raise DatabaseException("Playlist %s does not contain %s" % (self, track))
+
+class PhotoDatabase:
+    """An iTunes Photo database"""
+    def __init__(self, mountpoint="/mnt/ipod"):
+        """Create a Photo database object"""
+        self._itdb = gpod.itdb_photodb_parse(mountpoint, None)
+        if self._itdb == None:
+            self._itdb = gpod.itdb_photodb_create(mountpoint)
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "<PhotoDatabase Mountpoint:%s Albums:%s Photos:%s>" % (
+            repr(self.device['mountpoint']),
+            gpod.sw_get_list_len(self._itdb.photoalbums),
+            len(self))
+
+    def close(self):
+        gpod.itdb_photodb_write(self._itdb, None)
+        gpod.itdb_photodb_free(self._itdb)
+
+    def __len__(self):
+        return gpod.sw_get_list_len(self._itdb.photos)
+
+    def __getitem__(self, index):
+        if type(index) == types.SliceType:
+            return [self[i] for i in xrange(*index.indices(len(self)))]
+        else:
+            if index < 0:
+                index += len(self)
+            return Photo(proxied_photo=gpod.sw_get_photo(self._itdb.photos, index),
+                         ownerdb=self)
+
+    def new_PhotoAlbum(self,**kwargs):
+        """Create a new PhotoAlbum.
+        """
+        album = PhotoAlbum(self, **kwargs)
+        return album
+
+    def new_Photo(self,**kwargs):
+        """Create a new Photo.
+        """
+        kwargs['ownerdb'] = self
+        photo = Photo(**kwargs)
+        return photo
+
+    def remove(self, item):
+        """Remove a photo or album from a database.
+
+        item is either a Photo or PhotoAlbum object.
+        """
+
+        if isinstance(item, PhotoAlbum):
+            gpod.itdb_photodb_photoalbum_remove(self._itdb, item._pa, False)
+        elif isinstance(item, Photo):
+            gpod.itdb_photodb_remove_photo(self._itdb, None, item._photo)
+        else:
+            raise DatabaseException("Unable to remove a %s from database" % type(item))
+
+    def get_device(self):
+        return gpod.sw_ipod_device_to_dict(self._itdb.device)
+
+    def get_photoalbums(self):
+        """Get all photo albums."""
+        return _PhotoAlbums(self)
+
+    PhotoAlbums = property(get_photoalbums)
+    device      = property(get_device)
+
+class _PhotoAlbums:
+    def __init__(self, db):
+        self._db = db
+
+    def __len__(self):
+        return gpod.sw_get_list_len(self._db._itdb.photoalbums)
+
+    def __nonzero__(self):
+        return True
+
+    def __getitem__(self, index):
+        if type(index) == types.SliceType:
+            return [self[i] for i in xrange(*index.indices(len(self)))]
+        else:
+            if index < 0:
+                index += len(self)
+            return PhotoAlbum(self._db,
+                              proxied_photoalbum=gpod.sw_get_photoalbum(self._db._itdb.photoalbums,
+                                                                        index))
+
+    def __repr__(self):
+        return "<PhotoAlbums from %s>" % self._db
+
+    def __call__(self, name):
+        if type(name) in (types.TupleType, types.ListType):
+            return [self.__call__(name=i) for i in name]
+        else:
+            pa = gpod.itdb_photodb_photoalbum_by_name(self._db._itdb,
+                                                      name)
+            if pa:
+                return PhotoAlbum(self._db,
+                                  proxied_playlist=pa)
+            else:
+                raise KeyError("Album with name %s not found." % repr(name))
+
+
+class PhotoAlbum:
+    """A Photo Album in an iTunes database."""
+
+    def __init__(self, parent_db, title="New Album",
+                 pos=-1, proxied_photoalbum=None):
+
+        self._db = parent_db
+        if proxied_photoalbum:
+            self._pa = proxied_photoalbum
+        else:
+            self._pa = gpod.itdb_photodb_photoalbum_create(self._db._itdb, title, pos)
+
+    def add(self, photo):
+        """Add photo to photo album."""
+        gpod.itdb_photodb_photoalbum_add_photo(self._db._itdb, self._pa, photo._photo, -1)
+
+    def remove(self, photo):
+        gpod.itdb_photodb_remove_photo(self._db._itdb, self._pa, photo._photo)
+
+    def get_name(self):
+        """Get the name of the photo album."""
+        return self._pa.name
+
+    def set_name(self, name):
+        """Set the name for the photo album."""
+        self._pa.name = name
+
+    def get_album_type(self):
+        return self._pa.album_type
+
+    name       = property(get_name, set_name)
+    album_type = property(get_album_type)
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "<PhotoAlbum Title:%s Photos:%d Type:%d>" % (
+            repr(self.name),
+            len(self),
+            self.album_type)
+
+    def __getitem__(self, index):
+        if type(index) == types.SliceType:
+            return [self[i] for i in xrange(*index.indices(len(self)))]
+        else:
+            if index < 0:
+                index += len(self)
+            return Photo(proxied_photo=gpod.sw_get_photo(self._pa.members, index),
+                         ownerdb=self._db)
+
+    def __len__(self):
+        return gpod.sw_get_list_len(self._pa.members)
+
+    def __nonzero__(self):
+        return True
+
+class Photo:
+    """A photo in an iTunes Photo database."""
+
+    _proxied_attributes = [k for k in gpod._Itdb_Artwork.__dict__.keys()
+                            if not (k.startswith("_") or k.startswith("reserved"))]
+
+    def __init__(self, filename=None,
+                 proxied_photo=None, ownerdb=None):
+        """Create a Photo object."""
+        error = None
+
+        if filename:
+            self._photo = gpod.itdb_photodb_add_photo(ownerdb._itdb, filename, -1, 0, error)
+            self._database = ownerdb
+        elif proxied_photo:
+            self._photo = proxied_photo
+            self._database = ownerdb
+        else:
+            self._photo = gpod.itdb_artwork_new()
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "<Photo ID:%s Creation:'%s' Digitized:'%s' Size:%s>" % (
+            repr(self['id']),
+            self['creation_date'].strftime("%c"),
+            self['digitized_date'].strftime("%c"),
+            repr(self['artwork_size']))
+
+    def keys(self):
+        return list(self._proxied_attributes)
+
+    def items(self):
+        return [self[k] for k in self._proxied_attributes]
+
+    def pairs(self):
+        return [(k, self[k]) for k in self._proxied_attributes]
+
+    def __getitem__(self, item):
+        if item in self._proxied_attributes:
+            attr = getattr(self._photo, item)
+            if item.endswith("_date"):
+                try:
+                    return datetime.datetime.fromtimestamp(attr)
+                except:
+                    return datetime.datetime.fromtimestamp(0)
+            else:
+                return attr
+        else:
+            raise KeyError('No such key: %s' % item)
+
+    def __setitem__(self, item, value):
+        if type(value) == types.UnicodeType:
+            value = value.encode('UTF-8','replace')
+        if item in self._proxied_attributes:
+            return setattr(self._photo, item, value)
+        else:
+            raise KeyError('No such key: %s' % item)
+
+    def get_thumbnails(self):
+        return [Thumbnail(proxied_thumbnail=t,
+                          ownerobject=self) for t in gpod.sw_get_artwork_thumbnails(
+            self._photo)]
+
+    thumbnails = property(get_thumbnails)
+
+class Thumbnail:
+    """A thumbnail in an Photo."""
+
+    _proxied_attributes = [k for k in gpod._Itdb_Thumb.__dict__.keys()
+                            if not (k.startswith("_") or k.startswith("reserved"))]
+
+    def __init__(self, proxied_thumbnail=None, ownerobject=None):
+        """Create a thumbnail object."""
+
+        if not proxied_thumbnail:
+            raise NotImplemented("Can't create new Thumbnails from scratch, create Photos instead")
+
+        self._thumbnail = proxied_thumbnail
+        self.__ownerobject = ownerobject
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "<Thumbnail Filename:%s Size:%d Width:%d Height:%d>" % (
+            repr(self['filename']),
+            self['size'],
+            self['width'],
+            self['height'])
+
+    def keys(self):
+        return list(self._proxied_attributes)
+
+    def items(self):
+        return [self[k] for k in self._proxied_attributes]
+
+    def pairs(self):
+        return [(k, self[k]) for k in self._proxied_attributes]
+
+    def __getitem__(self, item):
+        if item in self._proxied_attributes:
+            return getattr(self._thumbnail, item)
+        else:
+            raise KeyError('No such key: %s' % item)
+
+    def __setitem__(self, item, value):
+        if type(value) == types.UnicodeType:
+            value = value.encode('UTF-8','replace')
+        if item in self._proxied_attributes:
+            return setattr(self._thumbnail, item, value)
+        else:
+            raise KeyError('No such key: %s' % item)
+
+    if pixbuf_support:
+        def get_pixbuf(self):
+            # this deals with coverart and photo albums
+            if hasattr(self.__ownerobject._database,"_itdb"):
+                return gpod.itdb_thumb_get_gdk_pixbuf(
+                    self.__ownerobject._database._itdb.device,
+                    self._thumbnail)
+            else:
+                return gpod.itdb_thumb_get_gdk_pixbuf(
+                    self.__ownerobject._database.device,
+                    self._thumbnail)
