@@ -1,23 +1,23 @@
 /*
- *  Copyright (C) 2005 Christophe Fergeau
+ *  Copyright (C) 2005-2007 Christophe Fergeau
+ *
  *
- * 
  *  The code contained in this file is free software; you can redistribute
  *  it and/or modify it under the terms of the GNU Lesser General Public
  *  License as published by the Free Software Foundation; either version
  *  2.1 of the License, or (at your option) any later version.
- *  
+ *
  *  This file is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  Lesser General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this code; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
+ *
  *  iTunes and iPod are trademarks of Apple
- * 
+ *
  *  This product is not supported/written/published by Apple!
  *
  */
@@ -43,21 +43,16 @@
 #include <stdio.h>
 #include <sys/types.h>
 
-/* FIXME: Writing aborts if a file exceeds the following size because
-   the mremap() further down will fail (at least most of the time on
-   most systems). As a workaround the size was increased from 2 MB to
-   16 MB. */
-#define IPOD_MMAP_SIZE 16 * 1024 * 1024
+#define DEFAULT_GSTRING_SIZE 128*1024
 
-struct iPodMmapBuffer {
-	int fd;
-	void *mmap_area;
-	size_t size;
+struct iPodSharedDataBuffer {
+	GString *data;
+	char *filename;
 	int ref_count;
 };
 
 struct _iPodBuffer {
-	struct iPodMmapBuffer *mmap;
+	struct iPodSharedDataBuffer *shared;
 	off_t offset;
 	guint byte_order;
 	DbType db_type;
@@ -65,101 +60,50 @@ struct _iPodBuffer {
 
 typedef struct _iPodBuffer iPodBuffer;
 
-static void
-ipod_mmap_buffer_destroy (struct iPodMmapBuffer *buf)
+static gboolean
+ipod_gstring_flush (struct iPodSharedDataBuffer *shared, GError **error)
 {
-	munmap (buf->mmap_area, buf->size);
-	close (buf->fd);
-	g_free (buf);
+	gboolean result;
+
+	result = g_file_set_contents (shared->filename, 
+				      shared->data->str, shared->data->len,
+				      error);
+	if (!result) {
+		return FALSE;
+	}
+	g_string_free (shared->data, TRUE);
+	g_free (shared->filename);
+	g_free (shared);
+
+	return TRUE;
 }
 
 static void
 ipod_buffer_destroy (iPodBuffer *buffer)
 {
-	buffer->mmap->ref_count--;
-	if (buffer->mmap->ref_count == 0) {
-		ipod_mmap_buffer_destroy (buffer->mmap);
+	buffer->shared->ref_count--;
+	if (buffer->shared->ref_count == 0) {
+		ipod_gstring_flush (buffer->shared, NULL);
 	}
 	g_free (buffer);
 }
 
 
 static void *
-ipod_buffer_get_pointer (iPodBuffer *buffer) 
+ipod_buffer_get_pointer (iPodBuffer *buffer)
 {
-	return &((unsigned char *)buffer->mmap->mmap_area)[buffer->offset];
+	if (buffer->shared->data->str == NULL) {
+		return NULL;
+	}
+	g_assert (buffer->offset < buffer->shared->data->len);
+	return &((unsigned char *)buffer->shared->data->str)[buffer->offset];
 }
 
-static int
-ipod_buffer_grow_file (struct iPodMmapBuffer *mmap_buf, off_t new_size)
+static void
+ipod_buffer_maybe_grow (iPodBuffer *buffer, off_t size)
 {
-	int result;
-
-	result = lseek (mmap_buf->fd, new_size, SEEK_SET);
-	if (result == (off_t)-1) {
-		g_print ("Failed to grow file to %lu: %s\n", 
-                        (unsigned long)new_size, strerror (errno));
-		return -1;
-	}
-	result = 0;
-	result = write (mmap_buf->fd, &result, 1);
-	if (result != 1) {
-		g_print ("Failed to write a byte at %lu: %s\n", 
-                        (unsigned long)new_size, strerror (errno));
-		return -1;
-	}
-
-	return 0;
-}
-
-
-static void * 
-ipod_buffer_grow_mapping (iPodBuffer *buffer, size_t size)
-{
-	void *new_address;
-#ifdef HAVE_MREMAP
-	
-	new_address = mremap (buffer->mmap->mmap_area, buffer->mmap->size,
-			      buffer->mmap->size + size, 0);
-#else
-	munmap (buffer->mmap->mmap_area, buffer->mmap->size);
-	new_address = mmap (buffer->mmap->mmap_area, buffer->mmap->size + size,
-			    PROT_READ | PROT_WRITE, MAP_SHARED, 
-			    buffer->mmap->fd, 0);
-#endif
-	/* Don't allow libc to move the current mapping since this would
-	 * force us to be very careful wrt pointers in the rest of the code
-	 */
-	if (new_address != buffer->mmap->mmap_area) {
-		return MAP_FAILED;
-	}
-
-	return new_address;
-}
-
-
-static int
-ipod_buffer_maybe_grow (iPodBuffer *buffer, off_t offset)
-{
-	void *new_address;
-
-	if (buffer->offset + offset <= buffer->mmap->size) {
-		return 0;
-	}
-
-	new_address = ipod_buffer_grow_mapping (buffer, IPOD_MMAP_SIZE);
-	if (new_address == MAP_FAILED) {
-		g_print ("Failed to mremap buffer\n");
-		return -1;
-	}
-
-	if (ipod_buffer_grow_file (buffer->mmap, 
-				   buffer->mmap->size + IPOD_MMAP_SIZE) != 0) {
-		return -1;
-	}
-	buffer->mmap->size += IPOD_MMAP_SIZE;
-
-	return 0;
+	g_string_set_size (buffer->shared->data, 
+			   buffer->shared->data->len + size);
 }
 
 static iPodBuffer *
@@ -167,19 +111,18 @@ ipod_buffer_get_sub_buffer (iPodBuffer *buffer, off_t offset)
 {
 	iPodBuffer *sub_buffer;
 
-	if (ipod_buffer_maybe_grow (buffer, offset) != 0) {
-		return NULL;
-	}
+	g_assert (buffer->offset + offset <= buffer->shared->data->len);
+
 	sub_buffer = g_new0 (iPodBuffer, 1);
 	if (sub_buffer == NULL) {
 		return NULL;
 	}
-	sub_buffer->mmap = buffer->mmap;
+	sub_buffer->shared = buffer->shared;
 	sub_buffer->offset = buffer->offset + offset;
 	sub_buffer->byte_order = buffer->byte_order;
 	sub_buffer->db_type = buffer->db_type;
 
-	buffer->mmap->ref_count++;
+	buffer->shared->ref_count++;
 
 	return sub_buffer;
 }
@@ -187,49 +130,25 @@ ipod_buffer_get_sub_buffer (iPodBuffer *buffer, off_t offset)
 static iPodBuffer *
 ipod_buffer_new (const char *filename, guint byte_order, DbType db_type)
 {
-	int fd;
-	struct iPodMmapBuffer *mmap_buf;
+	struct iPodSharedDataBuffer *shared;
 	iPodBuffer *buffer;
-	void *mmap_area;
 
-	fd = open (filename, O_RDWR | O_CREAT | O_TRUNC, 
-		   S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-	if (fd == -1) {
-		g_print ("Failed to open %s: %s\n", 
-			 filename, strerror (errno));
+	shared = g_new0 (struct iPodSharedDataBuffer, 1);
+	if (shared == NULL) {
 		return NULL;
 	}
-	
-	mmap_area = mmap (0, IPOD_MMAP_SIZE, PROT_READ | PROT_WRITE,
-			  MAP_SHARED, fd, 0);
-	if (mmap_area == MAP_FAILED) {
-		g_print ("Failed to mmap %s in memory: %s\n", filename, 
-			 strerror (errno));
-		close (fd);
-		return NULL;
-	}
-	mmap_buf = g_new0 (struct iPodMmapBuffer, 1);
-	if (mmap_buf == NULL) {
-		munmap (mmap_area, IPOD_MMAP_SIZE);
-		close (fd);
-		return NULL;
-	}
-	mmap_buf->mmap_area = mmap_area;
-	mmap_buf->size = IPOD_MMAP_SIZE;
-	mmap_buf->ref_count = 1;
-	mmap_buf->fd = fd;
-
-	if (ipod_buffer_grow_file (mmap_buf, IPOD_MMAP_SIZE) != 0) {
-		ipod_mmap_buffer_destroy (mmap_buf);
-		return  NULL;
-	}
+	shared->filename = g_strdup (filename);
+	shared->data = g_string_sized_new (DEFAULT_GSTRING_SIZE);
+	shared->ref_count = 1;
 
 	buffer = g_new0 (iPodBuffer, 1);
 	if (buffer == NULL) {
-		ipod_mmap_buffer_destroy (mmap_buf);
+		g_free (shared->filename);
+		g_string_free (shared->data, TRUE);
+		g_free (shared);
 		return NULL;
 	}
-	buffer->mmap = mmap_buf;
+	buffer->shared = shared;
 	buffer->byte_order = byte_order;
 	buffer->db_type = db_type;
 
@@ -242,19 +161,11 @@ enum MhsdType {
 	MHSD_TYPE_MHLF = 3
 };
 
-enum iPodThumbnailType {
-	IPOD_THUMBNAIL_FULL_SIZE = 0,
-	IPOD_THUMBNAIL_NOW_PLAYING = 1,
-	IPOD_PHOTO_LITTLE = 3,
-	IPOD_PHOTO_FULL_SIZE = 4
-};
-
-
 #define RETURN_SIZE_FOR(id, size) if (strncmp (id, header_id, 4) == 0) return (size)
 
 
 
-/* Returns the "real" size for a header, ie the size iTunes uses for it 
+/* Returns the "real" size for a header, ie the size iTunes uses for it
  * (padding included)
  */
 static int
@@ -285,10 +196,12 @@ init_header (iPodBuffer *buffer, gchar _header_id[4], guint header_len)
 		header_len = padded_size;
 	}
 	g_assert (header_len > sizeof (MHeader));
-	if (ipod_buffer_maybe_grow (buffer, header_len) != 0) {
+	ipod_buffer_maybe_grow (buffer, header_len);
+
+	mh = (MHeader*)ipod_buffer_get_pointer (buffer);
+	if (mh == NULL) {
 		return NULL;
 	}
-	mh = (MHeader*)ipod_buffer_get_pointer (buffer);
 	memset (mh, 0, header_len);
 
 	header_id = g_strndup (_header_id, 4);
@@ -303,7 +216,7 @@ init_header (iPodBuffer *buffer, gchar _header_id[4], guint header_len)
 }
 
 
-static int 
+static int
 write_mhod_type_1 (gchar *string, iPodBuffer *buffer)
 {
 	MhodHeaderArtworkType1 *mhod;
@@ -313,14 +226,14 @@ write_mhod_type_1 (gchar *string, iPodBuffer *buffer)
 
 	g_assert (string != NULL);
 
-	mhod = (MhodHeaderArtworkType1 *)init_header (buffer, "mhod", 
+	mhod = (MhodHeaderArtworkType1 *)init_header (buffer, "mhod",
 						      sizeof (MhodHeaderArtworkType1));
 	if (mhod == NULL) {
 		return -1;
 	}
 	total_bytes = sizeof (MhodHeaderArtworkType1);
 	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
-	/* Modify header length, since iTunes only puts the length of 
+	/* Modify header length, since iTunes only puts the length of
 	 * MhodHeader in header_len
 	 */
 	mhod->header_len = get_gint32 (sizeof (ArtworkDB_MhodHeader), buffer->byte_order);
@@ -329,50 +242,53 @@ write_mhod_type_1 (gchar *string, iPodBuffer *buffer)
 	mhod->string_len = get_gint32 (len, buffer->byte_order);
 
 	padding = 4 - ( (total_bytes + len) % 4 );
+	if (padding == 4)
+	    padding = 0;
 	mhod->padding = padding;
-	mhod->type = get_gint32 (0x0001, buffer->byte_order);
+	mhod->type = get_gint16 (0x01, buffer->byte_order);
 
 	/* Make sure we have enough free space to write the string */
-	if (ipod_buffer_maybe_grow (buffer, total_bytes + len + padding ) != 0) {
-		return  -1;
+	ipod_buffer_maybe_grow (buffer, len + padding);
+	mhod = ipod_buffer_get_pointer (buffer);
+	if (mhod == NULL) {
+		return -1;
 	}
 	memcpy (mhod->string, string, len);
 	total_bytes += len + padding;
-	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);	
+	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
 
 	dump_mhod_type_1 (mhod);
 
 	return total_bytes;
 }
 
-static int 
+static int
 write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
 {
 	ArtworkDB_MhodHeaderArtworkType3 *mhod;
 	unsigned int total_bytes;
-	int len;
+	glong len;
+	const gint g2l = sizeof (gunichar2);
 	gunichar2 *utf16, *strp;
 	int i, padding;
 
 	g_assert (string != NULL);
 
 	mhod = (ArtworkDB_MhodHeaderArtworkType3 *)
-	    init_header (buffer, "mhod", 
+	    init_header (buffer, "mhod",
 			 sizeof (ArtworkDB_MhodHeaderArtworkType3));
 	if (mhod == NULL) {
 		return -1;
 	}
 	total_bytes = sizeof (ArtworkDB_MhodHeaderArtworkType3);
 	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
-	/* Modify header length, since iTunes only puts the length of 
+	/* Modify header length, since iTunes only puts the length of
 	 * MhodHeader in header_len
 	 */
-	mhod->header_len = get_gint32 (sizeof (ArtworkDB_MhodHeader), 
+	mhod->header_len = get_gint32 (sizeof (ArtworkDB_MhodHeader),
 				       buffer->byte_order);
 	mhod->type = get_gint16 (3, buffer->byte_order);
 
-	len = strlen (string);
-
 	/* FIXME: Tidy this up, combine cases more */
 	/* Some magic: endianess-reversed (BE) mobile phones use UTF8
 	 * (version 1) with padding, standard iPods (LE) use UTF16
@@ -380,40 +296,49 @@ write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
 	switch (buffer->byte_order)
 	{
 	case G_LITTLE_ENDIAN:
-	    mhod->mhod_version = 2;
-	    /* number of bytes of the string encoded in UTF-16 */
-	    mhod->string_len = get_gint32 (2*len, buffer->byte_order);
-	    padding = 4 - ( (total_bytes + 2*len) % 4 );
-	    mhod->padding = padding; /* high byte is padding length (0-3) */
-
-	    /* Make sure we have enough free space to write the string */
-	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
-		return  -1;
-	    }
-
-	    utf16 = g_utf8_to_utf16 (string, -1, NULL, NULL, NULL);
-	    if (utf16 == NULL) {		
+	    utf16 = g_utf8_to_utf16 (string, -1, NULL, &len, NULL);
+	    if (utf16 == NULL) {
 		return -1;
 	    }
-	    strp = (gunichar2 *)mhod->string;
-	    memcpy (strp, utf16, 2*len);
-	    g_free (utf16);
-	    for (i = 0; i < len; i++) {
-		strp[i] = get_gint16 (strp[i], buffer->byte_order);
+
+	    mhod->mhod_version = 2;
+	    /* number of bytes of the string encoded in UTF-16 */
+	    mhod->string_len = get_gint32 (g2l * len, buffer->byte_order);
+	    padding = 4 - ( (total_bytes + g2l*len) % 4 );
+	    if (padding == 4)
+		padding = 0;
+	    mhod->padding = padding;
+ 	    total_bytes += g2l*len + padding;
+
+	    /* Make sure we have enough free space to write the string */
+	    ipod_buffer_maybe_grow (buffer, g2l*len + padding);
+	    mhod = ipod_buffer_get_pointer (buffer);
+	    if (mhod == NULL) {
+		    g_free (utf16);
+		    return  -1;
 	    }
-	    total_bytes += 2*len+padding;
+	    strp = (gunichar2 *)mhod->string;
+	    for (i = 0; i < len; i++) {
+		strp[i] = get_gint16 (utf16[i], buffer->byte_order);
+	    }
+	    g_free (utf16);
 	    break;
 	case G_BIG_ENDIAN:
 	    mhod->mhod_version = 1;
+            /* FIXME: len isn't initialized */
 	    mhod->string_len = get_gint32 (len, buffer->byte_order);
 	    /* pad string if necessary */
 	    /* e.g. len = 7 bytes, len%4 = 3, 4-3=1 -> requires 1 byte
 	       padding */
 	    padding = 4 - ( (total_bytes + len) % 4 );
-	    mhod->padding = padding; /* high byte is padding length (0-3) */
+	    if (padding == 4)
+		padding = 0;
+	    mhod->padding = padding;
 	    /* Make sure we have enough free space to write the string */
-	    if (ipod_buffer_maybe_grow (buffer, total_bytes + 2*len+padding) != 0) {
-		return  -1;
+	    ipod_buffer_maybe_grow (buffer, len+padding);
+	    mhod = ipod_buffer_get_pointer (buffer);
+	    if (mhod == NULL) {
+		    return -1;
 	    }
 	    memcpy (mhod->string, string, len);
 	    total_bytes += (len+padding);
@@ -425,7 +350,7 @@ write_mhod_type_3 (gchar *string, iPodBuffer *buffer)
 	return total_bytes;
 }
 
-static int 
+static int
 write_mhni (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buffer)
 {
 	MhniHeader *mhni;
@@ -437,7 +362,7 @@ write_mhni (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buff
 		return -1;
 	}
 
-	mhni = (MhniHeader *)init_header (buffer, "mhni", 
+	mhni = (MhniHeader *)init_header (buffer, "mhni",
 					  sizeof (MhniHeader));
 	if (mhni == NULL) {
 		return -1;
@@ -456,11 +381,10 @@ write_mhni (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buff
 					   buffer->byte_order);
 	mhni->ithmb_offset =   get_gint32 (thumb->offset,
 					   buffer->byte_order);
-
-	mhni->vertical_padding =
-	    get_gint16 (thumb->vertical_padding, buffer->byte_order);
-	mhni->horizontal_padding = 
-	    get_gint16 (thumb->horizontal_padding, buffer->byte_order);
+	mhni->vertical_padding = get_gint16 (thumb->vertical_padding,
+					     buffer->byte_order);
+	mhni->horizontal_padding = get_gint16 (thumb->horizontal_padding,
+					       buffer->byte_order);
 
 	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 	if (sub_buffer == NULL) {
@@ -472,12 +396,13 @@ write_mhni (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buff
 		return -1;
 	}
 	total_bytes += bytes_written;
+	mhni = ipod_buffer_get_pointer (buffer);
 	mhni->total_len = get_gint32 (total_bytes, buffer->byte_order);
 	/* Only update number of children when all went well to try to get
-	 * something somewhat consistent when there are errors 
+	 * something somewhat consistent when there are errors
 	 */
 	mhni->num_children = get_gint32 (1, buffer->byte_order);
-	
+
 	dump_mhni (mhni);
 
 	return total_bytes;
@@ -496,7 +421,7 @@ write_mhod (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buff
 	}
 
 	mhod = (ArtworkDB_MhodHeader *)
-	    init_header (buffer, "mhod", 
+	    init_header (buffer, "mhod",
 			 sizeof (ArtworkDB_MhodHeader));
 	if (mhod == NULL) {
 		return -1;
@@ -514,6 +439,7 @@ write_mhod (Itdb_DB *db, Itdb_Thumb *thumb, int correlation_id, iPodBuffer *buff
 		return -1;
 	}
 	total_bytes += bytes_written;
+	mhod = ipod_buffer_get_pointer (buffer);
 	mhod->total_len = get_gint32 (total_bytes, buffer->byte_order);
 
 	dump_mhod (mhod);
@@ -531,6 +457,8 @@ write_mhii (Itdb_DB *db, void *data, iPodBuffer *buffer)
 	GList *it = NULL;
 	Itdb_Track *song;
 	Itdb_Artwork *artwork;
+	guint64 mactime;
+	Itdb_Device *device = db_get_device (db);
 
 	mhii = (MhiiHeader *)init_header (buffer, "mhii", sizeof (MhiiHeader));
 	if (mhii == NULL) {
@@ -541,51 +469,59 @@ write_mhii (Itdb_DB *db, void *data, iPodBuffer *buffer)
 	switch( buffer->db_type) {
 	case DB_TYPE_ITUNES:
 		song = (Itdb_Track *)data;
+		artwork = song->artwork;
 		mhii->song_id = get_gint64 (song->dbid, buffer->byte_order);
-		mhii->image_id = get_guint32 (song->artwork->id, buffer->byte_order);
-		mhii->orig_img_size = get_gint32 (song->artwork->artwork_size, 
-				buffer->byte_order);
-		it = song->artwork->thumbnails;
 		break;
 	case DB_TYPE_PHOTO:
 		artwork = (Itdb_Artwork *)data;
 		mhii->song_id = get_gint64 (artwork->id + 2, buffer->byte_order);
-		mhii->image_id = get_guint32 (artwork->id, buffer->byte_order);
-		mhii->orig_date = get_guint32 (itdb_time_host_to_mac( artwork->creation_date )
-				, buffer->byte_order);
-		mhii->digitised_date = get_guint32 (itdb_time_host_to_mac( artwork->creation_date )
-				, buffer->byte_order);
-		it = artwork->thumbnails;
 		break;
+	default:
+	        g_return_val_if_reached (-1);
 	}
+	mhii->image_id = get_guint32 (artwork->id, buffer->byte_order);
+	mhii->unknown4 = get_gint32 (artwork->unk028, buffer->byte_order);
+	mhii->rating = get_gint32 (artwork->rating, buffer->byte_order);
+	mhii->unknown6 = get_gint32 (artwork->unk036, buffer->byte_order);
+
+	mactime = device_time_time_t_to_mac (device, artwork->creation_date);
+	mhii->orig_date = get_guint32 (mactime, buffer->byte_order);
+
+	mactime = device_time_time_t_to_mac (device, artwork->digitized_date);
+	mhii->digitized_date = get_guint32 (mactime, buffer->byte_order);
+
+	mhii->orig_img_size = get_gint32 (artwork->artwork_size, buffer->byte_order);
 	num_children = 0;
-	while (it != NULL) {
+	for (it=artwork->thumbnails; it!=NULL; it=it->next)
+	{
 		iPodBuffer *sub_buffer;
 		Itdb_Thumb *thumb;
 		const Itdb_ArtworkFormat *img_info;
 
-		mhii->num_children = get_gint32 (num_children, 
+		thumb = (Itdb_Thumb *)it->data;
+		img_info = itdb_get_artwork_info_from_type (
+		    db_get_device(db), thumb->type);
+		if (img_info == NULL) {
+		    /* skip this thumb */
+		    continue;
+		}
+
+		mhii->num_children = get_gint32 (num_children,
 						 buffer->byte_order);
 		mhii->total_len = get_gint32 (total_bytes, buffer->byte_order);
 		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 		if (sub_buffer == NULL) {
 			return -1;
 		}
-		thumb = (Itdb_Thumb *)it->data;
-		img_info = itdb_get_artwork_info_from_type (
-		    db_get_device(db), thumb->type);
-		if (img_info == NULL) {
-			return -1;
-		}
-		bytes_written = write_mhod (db, thumb, img_info->correlation_id, 
+		bytes_written = write_mhod (db, thumb, img_info->correlation_id,
 					    sub_buffer);
 		ipod_buffer_destroy (sub_buffer);
 		if (bytes_written == -1) {
 			return -1;
 		}
 		total_bytes += bytes_written;
+		mhii = ipod_buffer_get_pointer (buffer);
 		num_children++;
-		it = it->next;
 	}
 
 	mhii->num_children = get_gint32 (num_children, buffer->byte_order);
@@ -625,10 +561,9 @@ write_mhli (Itdb_DB *db, iPodBuffer *buffer )
 		Itdb_Track *song;
 		int bytes_written;
 		iPodBuffer *sub_buffer;
-
 		if (buffer->db_type == DB_TYPE_ITUNES) {
 			song = (Itdb_Track*)it->data;
-			if (song->artwork->id == 0) {
+			if (!song->artwork || (song->artwork->id == 0)) {
 				it = it->next;
 				continue;
 			}
@@ -642,9 +577,10 @@ write_mhli (Itdb_DB *db, iPodBuffer *buffer )
 		if (bytes_written != -1) {
 			num_thumbs++;
 			total_bytes += bytes_written;
-		} 
+		}
 		it = it->next;
 	}
+	mhli = ipod_buffer_get_pointer (buffer);
 	mhli->num_children = get_gint32 (num_thumbs, buffer->byte_order);
 	dump_mhl ((MhlHeader *)mhli, "mhli");
 
@@ -671,7 +607,7 @@ write_mhia (gint image_id, iPodBuffer *buffer)
 }
 
 static int
-write_mhba (Itdb_PhotoAlbum *photo_album, iPodBuffer *buffer)
+write_mhba (Itdb_PhotoAlbum *album, iPodBuffer *buffer)
 {
 	GList *it;
 	MhbaHeader *mhba;
@@ -684,10 +620,27 @@ write_mhba (Itdb_PhotoAlbum *photo_album, iPodBuffer *buffer)
 		return -1;
 	}
 	mhba->num_mhods = get_gint32(1, buffer->byte_order);
-	mhba->playlist_id = get_gint32(photo_album->album_id, buffer->byte_order);
-	mhba->master = get_gint32(photo_album->master, buffer->byte_order);
-	mhba->prev_playlist_id = get_gint32(photo_album->prev_album_id, buffer->byte_order);
-	mhba->num_mhias = get_gint32(photo_album->num_images, buffer->byte_order);
+	mhba->num_mhias = get_gint32(g_list_length (album->members),
+				     buffer->byte_order);
+	mhba->album_id = get_gint32(album->album_id, buffer->byte_order);
+	mhba->unk024 = get_gint32(album->unk024, buffer->byte_order);
+	mhba->unk028 = get_gint16(album->unk028, buffer->byte_order);
+	mhba->album_type = album->album_type;
+	mhba->playmusic = album->playmusic;
+	mhba->repeat = album->repeat;
+	mhba->random = album->random;
+	mhba->show_titles = album->show_titles;
+	mhba->transition_direction = album->transition_direction;
+	mhba->slide_duration = get_gint32(album->slide_duration,
+					  buffer->byte_order);
+	mhba->transition_duration = get_gint32(album->transition_duration,
+					       buffer->byte_order);
+	mhba->unk044 = get_gint32(album->unk044, buffer->byte_order);
+	mhba->unk048 = get_gint32(album->unk048, buffer->byte_order);
+	mhba->song_id = get_gint64(album->song_id, buffer->byte_order);
+	mhba->prev_album_id = get_gint32(album->prev_album_id,
+					 buffer->byte_order);
+
 	total_bytes = get_gint32 (mhba->header_len, buffer->byte_order);
 
 	/* FIXME: Write other mhods */
@@ -696,27 +649,30 @@ write_mhba (Itdb_PhotoAlbum *photo_album, iPodBuffer *buffer)
 	if (sub_buffer == NULL) {
 	    return -1;
 	}
-	bytes_written = write_mhod_type_1 (photo_album->name, sub_buffer);
+	bytes_written = write_mhod_type_1 (album->name, sub_buffer);
 	ipod_buffer_destroy (sub_buffer);
 	if (bytes_written == -1) {
 	    return -1;
-	} 
+	}
 	total_bytes += bytes_written;
+	mhba = ipod_buffer_get_pointer (buffer);
 
-	for (it = photo_album->members; it != NULL; it = it->next) {
-		gint image_id = GPOINTER_TO_INT(it->data);
+	for (it = album->members; it != NULL; it = it->next) {
+	        Itdb_Artwork *photo = it->data;
+		g_return_val_if_fail (photo, -1);
 
 		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 		if (sub_buffer == NULL) {
 		    return -1;
 		}
-		bytes_written = write_mhia (image_id, sub_buffer);
+		bytes_written = write_mhia (photo->id, sub_buffer);
 		ipod_buffer_destroy (sub_buffer);
 		if (bytes_written == -1) {
 		    return -1;
-		} 
+		}
 		total_bytes += bytes_written;
 	}
+	mhba = ipod_buffer_get_pointer (buffer);
 	mhba->total_len = get_gint32( total_bytes, buffer->byte_order );
 	dump_mhba ( mhba );
 	return total_bytes;
@@ -737,21 +693,24 @@ write_mhla (Itdb_DB *db, iPodBuffer *buffer)
 	total_bytes = get_gint32 (mhla->header_len, buffer->byte_order);
 	if (buffer->db_type == DB_TYPE_PHOTO) {
 	    unsigned int bytes_written;
-
+            unsigned int num_children = 0;
 	    for (it = db_get_photodb(db)->photoalbums; it != NULL; it = it->next) {
-		Itdb_PhotoAlbum *photo_album = (Itdb_PhotoAlbum *)it->data;
+		Itdb_PhotoAlbum *album = (Itdb_PhotoAlbum *)it->data;
 
 		sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
 		if (sub_buffer == NULL) {
 		    return -1;
 		}
-		bytes_written = write_mhba (photo_album, sub_buffer);
+		bytes_written = write_mhba (album, sub_buffer);
 		ipod_buffer_destroy (sub_buffer);
 		if (bytes_written == -1) {
 		    return -1;
-		} 
+		}
 		total_bytes += bytes_written;
-		mhla->num_children++;
+		mhla = ipod_buffer_get_pointer (buffer);
+		num_children++;
+		mhla->num_children = get_gint32 (num_children,
+                                                 buffer->byte_order);
 	    }
 	}
 
@@ -761,39 +720,56 @@ write_mhla (Itdb_DB *db, iPodBuffer *buffer)
 }
 
 static int
-write_mhif (Itdb_DB *db, iPodBuffer *buffer, enum iPodThumbnailType type)
+write_mhif (Itdb_DB *db, iPodBuffer *buffer,
+            const Itdb_ArtworkFormat *img_info)
 {
 	MhifHeader *mhif;
-	const Itdb_ArtworkFormat *img_info;
-	
+
 	mhif = (MhifHeader *)init_header (buffer, "mhif", sizeof (MhifHeader));
 	if (mhif == NULL) {
 		return -1;
 	}
 	mhif->total_len = mhif->header_len;
-	
-	img_info = itdb_get_artwork_info_from_type (db_get_device(db), type);
-	if (img_info == NULL) {
-		return -1;
-	}
-	mhif->correlation_id = get_gint32 (img_info->correlation_id, 
+
+	mhif->correlation_id = get_gint32 (img_info->correlation_id,
 					   buffer->byte_order);
 	mhif->image_size = get_gint32 (img_info->height * img_info->width * 2,
 				       buffer->byte_order);
 
 	dump_mhif (mhif);
-	
+
 	return get_gint32 (mhif->header_len, buffer->byte_order);
 }
 
+G_GNUC_INTERNAL gboolean
+itdb_thumb_type_is_valid_for_db (const ItdbThumbType thumb_type, DbType db_type)
+{
+        switch (thumb_type) {
+        case ITDB_THUMB_COVER_SMALL:
+        case ITDB_THUMB_COVER_LARGE:
+        case ITDB_THUMB_COVER_XLARGE:
+        case ITDB_THUMB_COVER_MEDIUM:
+        case ITDB_THUMB_COVER_SMEDIUM:
+        case ITDB_THUMB_COVER_XSMALL:
+	    return (db_type == DB_TYPE_ITUNES);
+        case ITDB_THUMB_PHOTO_SMALL:
+        case ITDB_THUMB_PHOTO_LARGE:
+        case ITDB_THUMB_PHOTO_FULL_SCREEN:
+        case ITDB_THUMB_PHOTO_TV_SCREEN:
+	    return (db_type == DB_TYPE_PHOTO);
+        }
+
+        g_return_val_if_reached (FALSE);
+}
+
 static int
 write_mhlf (Itdb_DB *db, iPodBuffer *buffer)
 {
 	MhlfHeader *mhlf;
 	unsigned int total_bytes;
 	int bytes_written;
-	iPodBuffer *sub_buffer;
-	enum iPodThumbnailType thumb_type = IPOD_THUMBNAIL_FULL_SIZE;
+        const Itdb_ArtworkFormat *formats; 
+        unsigned int num_children;
 
 	mhlf = (MhlfHeader *)init_header (buffer, "mhlf", sizeof (MhlfHeader));
 	if (mhlf == NULL) {
@@ -801,76 +777,48 @@ write_mhlf (Itdb_DB *db, iPodBuffer *buffer)
 	}
 
 	total_bytes = get_gint32 (mhlf->header_len, buffer->byte_order);
+        num_children = 0;
+        mhlf->num_files = get_gint32 (num_children, buffer->byte_order);
 
-	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
-	if (sub_buffer == NULL) {
-		return -1;
-	}
+        formats = itdb_device_get_artwork_formats (db_get_device(db));
+        if (formats == NULL) {
+                return total_bytes;
+        }
 
-	switch (buffer->db_type) {
-	case DB_TYPE_ITUNES:
-		thumb_type = IPOD_THUMBNAIL_FULL_SIZE;
-		break;
-	case DB_TYPE_PHOTO:
-		thumb_type = IPOD_PHOTO_FULL_SIZE;
-		break;
-	}
-	bytes_written = write_mhif (db
-				  , sub_buffer
-				  , thumb_type);
-	ipod_buffer_destroy (sub_buffer);
-	if (bytes_written == -1) {
-		return -1;
-	} 
-	total_bytes += bytes_written;
-	
-	/* Only update number of children when all went well to try to get
-	 * something somewhat consistent when there are errors 
-	 */
-	/* For the ArworkDB file, there are only 2 physical file storing
-	 * thumbnails: F1016_1.ithmb for the bigger thumbnails (39200 bytes)
-	 * and F1017_1.ithmb for the 'now playing' thumbnails (6272)
-	 */
-	mhlf->num_files = get_gint32 (1, buffer->byte_order);
+        while (formats->type != -1) {
+	        iPodBuffer *sub_buffer;
+                if (!itdb_thumb_type_is_valid_for_db (formats->type, buffer->db_type)) {
+                        formats++;
+                        continue;
+                }
+        	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
+        	if (sub_buffer == NULL) {
+        		return -1;
+        	}
 
-	sub_buffer = ipod_buffer_get_sub_buffer (buffer, total_bytes);
-	if (sub_buffer == NULL) {
-		return -1;
-	}
-
-	switch (buffer->db_type) {
-	case DB_TYPE_ITUNES:
-		thumb_type = IPOD_THUMBNAIL_NOW_PLAYING;
-		break;
-	case DB_TYPE_PHOTO:
-		thumb_type = IPOD_PHOTO_LITTLE;
-		break;
-	}
-	bytes_written = write_mhif (db
-				  , sub_buffer
-				  , thumb_type);
-	ipod_buffer_destroy (sub_buffer);
-	if (bytes_written == -1) {
-		return -1;
-	} 
-	total_bytes += bytes_written;
-
-	/* Only update number of children when all went well to try to get
-	 * something somewhat consistent when there are errors 
-	 */
-	/* For the ArworkDB file, there are only 2 physical file storing
-	 * thumbnails: F1016_1.ithmb for the bigger thumbnails (39200 bytes)
-	 * and F1017_1.ithmb for the 'now playing' thumbnails (6272)
-	 */
-	mhlf->num_files = get_gint32 (2, buffer->byte_order);
+        	bytes_written = write_mhif (db, sub_buffer, formats);
+	        			    
+        	ipod_buffer_destroy (sub_buffer);
+        	if (bytes_written == -1) {
+        		return -1;
+        	}
+        	total_bytes += bytes_written;
+		mhlf = ipod_buffer_get_pointer (buffer);
 
+                num_children++;
+        	/* Only update number of children when all went well to try 
+                 * to get something somewhat consistent when there are errors
+        	 */
+        	mhlf->num_files = get_gint32 (num_children, buffer->byte_order);
+                formats++;
+        }
 	dump_mhl ((MhlHeader *)mhlf, "mhlf");
 
 	return total_bytes;
 }
 
 
-static int 
+static int
 write_mhsd (Itdb_DB *db, iPodBuffer *buffer, enum MhsdType type)
 {
 	ArtworkDB_MhsdHeader *mhsd;
@@ -909,6 +857,7 @@ write_mhsd (Itdb_DB *db, iPodBuffer *buffer, enum MhsdType type)
 		return -1;
 	} else {
 		total_bytes += bytes_written;
+		mhsd = ipod_buffer_get_pointer (buffer);
 		mhsd->total_len = get_gint32 (total_bytes, buffer->byte_order);
 	}
 
@@ -925,6 +874,7 @@ write_mhfd (Itdb_DB *db, iPodBuffer *buffer, int id_max)
 	int bytes_written;
 	int i;
 
+	
 	mhfd = (MhfdHeader *)init_header (buffer, "mhfd", sizeof (MhfdHeader));
 	if (mhfd == NULL) {
 		return -1;
@@ -936,7 +886,7 @@ write_mhfd (Itdb_DB *db, iPodBuffer *buffer, int id_max)
 		mhfd->unknown2 = get_gint32 (2, buffer->byte_order);
 		break;
 	case DB_TYPE_ITUNES:
-		mhfd->unknown2 = get_gint32 (1, buffer->byte_order);
+		mhfd->unknown2 = get_gint32 (2, buffer->byte_order);
 		break;
 	}
 	mhfd->next_id = get_gint32 (id_max, buffer->byte_order);
@@ -950,10 +900,11 @@ write_mhfd (Itdb_DB *db, iPodBuffer *buffer, int id_max)
 		}
 		bytes_written = write_mhsd (db, sub_buffer, i);
 		ipod_buffer_destroy (sub_buffer);
-		if (bytes_written == -1) {			
+		if (bytes_written == -1) {
 			return -1;
 		}
 		total_bytes += bytes_written;
+		mhfd = ipod_buffer_get_pointer (buffer);
 		mhfd->total_len = get_gint32 (total_bytes, buffer->byte_order);
 		mhfd->num_children = get_gint32 (i, buffer->byte_order);
 	}
@@ -967,28 +918,96 @@ static unsigned int
 ipod_artwork_db_set_ids (Itdb_iTunesDB *db)
 {
 	GList *it;
-	unsigned int id;
-	
-	id = 64;
+	unsigned int max_id;
+
+	max_id = 0;
 	for (it = db->tracks; it != NULL; it = it->next) {
 		Itdb_Track *song;
 
 		song = (Itdb_Track *)it->data;
+		if (max_id <= song->id) {
+		  max_id = song->id;
+		}
 		if (song->artwork->thumbnails != NULL) {
-			song->artwork->id = id;
-			id++;
+			song->artwork->id = song->id;
 		}
 	}
-	
-	return id;
+
+	return max_id;
 }
 
+
+static void
+itdb_track_filter_thumbnails (Itdb_iTunesDB *itdb, Itdb_Track *track)
+{
+    const Itdb_ArtworkFormat *formats;
+    const Itdb_ArtworkFormat *format;
+    GList *it;
+    GList *supported_thumbs;
+
+    if (track->artwork == NULL) {
+        return;
+    }
+
+    if (track->artwork->thumbnails == NULL) {
+        return;
+    }
+
+    if (itdb->device == NULL) {
+        itdb_artwork_free (track->artwork);
+        track->artwork = NULL;
+        return;
+    }
+
+    formats = itdb_device_get_artwork_formats (itdb->device);
+    if (formats == NULL) {
+        itdb_artwork_free (track->artwork);
+        track->artwork = NULL;
+        return;
+    }
+
+    supported_thumbs = NULL;
+    for (format = formats; format->type != -1; format++) {
+        for (it = track->artwork->thumbnails; it != NULL; it = it->next) {
+            Itdb_Thumb *thumb = (Itdb_Thumb *)it->data;
+
+            if (thumb->type == format->type) {
+                supported_thumbs = g_list_prepend (supported_thumbs, it->data);
+                /* Remove element from the original list, there will
+                 * be less work next time we iterate over the thumbnails
+                 */
+                track->artwork->thumbnails = g_list_remove_link (track->artwork->thumbnails, it);
+                break;
+            }
+        }
+    }
+
+    if (supported_thumbs == NULL) {
+        itdb_artwork_free (track->artwork);
+        track->artwork = NULL;
+        return;
+    }
+    g_list_free (track->artwork->thumbnails);
+    track->artwork->thumbnails = supported_thumbs;
+}
+
+static void
+itdb_filter_thumbnails (Itdb_iTunesDB *itdb)
+{
+    GList *it;
+
+    for (it = itdb->tracks; it != NULL; it = it->next) {
+        itdb_track_filter_thumbnails (itdb, (Itdb_Track *)it->data);
+    }
+}
+
+
+
 int
 ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 {
 	iPodBuffer *buf;
 	int bytes_written;
-	int result;
 	char *filename;
 	int id_max;
 	Itdb_DB db;
@@ -996,6 +1015,9 @@ ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 	db.db_type = DB_TYPE_ITUNES;
 	db.db.itdb = itdb;
 
+
+        itdb_filter_thumbnails (itdb);
+
 	/* First, let's write the .ithmb files, this will create the
 	 * various thumbnails as well */
 
@@ -1014,33 +1036,22 @@ ipod_write_artwork_db (Itdb_iTunesDB *itdb)
 	buf = ipod_buffer_new (filename, itdb->device->byte_order, DB_TYPE_ITUNES);
 	if (buf == NULL) {
 		g_print ("Couldn't create %s\n", filename);
-		g_free (filename);
+	        g_free (filename);
 		return -1;
 	}
+	g_free (filename);
 	bytes_written = write_mhfd (&db, buf, id_max);
 
-	/* Refcount of the mmap buffer should drop to 0 and this should
+	/* Refcount of the shared buffer should drop to 0 and this should
 	 * sync buffered data to disk
-	 * FIXME: it's probably less error-prone to add a ipod_buffer_mmap_sync
-	 * call...
 	 */
 	ipod_buffer_destroy (buf);
 
 	if (bytes_written == -1) {
 		g_print ("Failed to save %s\n", filename);
-		g_free (filename);
 		/* FIXME: maybe should unlink the file we may have created */
 		return -1;
 	}
-
-	result = truncate (filename, bytes_written);
-	if (result != 0) {
-		g_print ("Failed to truncate %s: %s\n", 
-			 filename, strerror (errno));
-		g_free (filename);	
-		return -1;
-	}
-	g_free (filename);
 	return 0;
 }
 
@@ -1049,7 +1060,6 @@ ipod_write_photo_db (Itdb_PhotoDB *photodb)
 {
 	iPodBuffer *buf;
 	int bytes_written;
-	int result;
 	char *filename;
 	int id_max;
 	Itdb_DB db;
@@ -1073,28 +1083,17 @@ ipod_write_photo_db (Itdb_PhotoDB *photodb)
 	id_max = itdb_get_free_photo_id( photodb );
 	bytes_written = write_mhfd (&db, buf, id_max);
 
-	/* Refcount of the mmap buffer should drop to 0 and this should
+	/* Refcount of the shared buffer should drop to 0 and this should
 	 * sync buffered data to disk
-	 * FIXME: it's probably less error-prone to add a ipod_buffer_mmap_sync
-	 * call...
 	 */
 	ipod_buffer_destroy (buf);
+        g_free (filename);
 
 	if (bytes_written == -1) {
 		g_print ("Failed to save %s\n", filename);
-		g_free (filename);
 		/* FIXME: maybe should unlink the file we may have created */
 		return -1;
 	}
-
-	result = truncate (filename, bytes_written);
-	if (result != 0) {
-		g_print ("Failed to truncate %s: %s\n", 
-			 filename, strerror (errno));
-		g_free (filename);	
-		return -1;
-	}
-	g_free (filename);
 	return 0;
 }
 #else
