@@ -1,6 +1,5 @@
-/* Time-stamp: <2007-01-06 20:46:59 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_private.h,v 1.16 2007/01/08 10:23:15 jcsjcs Exp $
+|  $Id: itdb_private.h 1749 2007-10-30 08:21:16Z teuf $
 */
 
 #ifndef __ITDB_PRIVATE_H__
@@ -33,7 +32,6 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
-#include "glib-compat.h"
 #include "itdb_device.h"
 #include "itdb.h"
 
@@ -83,12 +81,12 @@ typedef struct
 struct playcount {
     guint32 playcount;
     guint32 skipped;     /* skipped (only for Shuffle's iTunesStats */
-    guint32 time_played;
+    time_t time_played;
     guint32 bookmark_time;
     gint32 rating;
     gint32 pc_unk16;     /* unknown field in Play Counts file */
     guint32 skipcount;
-    guint32 last_skipped;
+    time_t last_skipped;
     gint32 st_unk06;     /* unknown field in iTunesStats file */
     gint32 st_unk09;     /* unknown field in iTunesStats file */
 };
@@ -143,8 +141,8 @@ struct _Itdb_DB{
 
 typedef struct _Itdb_DB Itdb_DB;
 
-G_GNUC_INTERNAL gboolean itdb_spl_action_known (SPLAction action);
-G_GNUC_INTERNAL void itdb_splr_free (SPLRule *splr);
+G_GNUC_INTERNAL gboolean itdb_spl_action_known (ItdbSPLAction action);
+G_GNUC_INTERNAL void itdb_splr_free (Itdb_SPLRule *splr);
 G_GNUC_INTERNAL const gchar *itdb_photodb_get_mountpoint (Itdb_PhotoDB *photodb);
 G_GNUC_INTERNAL gchar *db_get_mountpoint (Itdb_DB *db);
 G_GNUC_INTERNAL Itdb_Device *db_get_device(Itdb_DB *db);
@@ -152,4 +150,10 @@ G_GNUC_INTERNAL gint itdb_get_free_photo_id ( Itdb_PhotoDB *db );
 G_GNUC_INTERNAL Itdb_iTunesDB *db_get_itunesdb (Itdb_DB *db);
 G_GNUC_INTERNAL Itdb_PhotoDB *db_get_photodb (Itdb_DB *db);
 G_GNUC_INTERNAL gint itdb_thumb_get_byteorder (ItdbThumbFormat format);
+G_GNUC_INTERNAL time_t device_time_mac_to_time_t (Itdb_Device *device, 
+						guint64 mactime);
+G_GNUC_INTERNAL guint64 device_time_time_t_to_mac (Itdb_Device *device,
+						 time_t timet);
+G_GNUC_INTERNAL gint itdb_musicdirs_number_by_mountpoint (const gchar *mountpoint);
+G_GNUC_INTERNAL gboolean itdb_device_requires_checksum (Itdb_Device *device);
 #endif
