@@ -1,6 +1,5 @@
-/* Time-stamp: <2006-11-23 23:32:16 jcs>
-|
-|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+/*
+|  Copyright (C) 2002-2007 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
 |  URL: http://www.gtkpod.org/
@@ -24,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: itdb_track.c,v 1.25 2007/01/13 20:24:50 tmzullinger Exp $
+|  $Id: itdb_track.c 1718 2007-10-05 19:20:01Z teuf $
 */
 
 #include <config.h>
@@ -248,14 +247,15 @@ void itdb_track_free (Itdb_Track *track)
     g_return_if_fail (track);
 
     g_free (track->title);
-    g_free (track->artist);
+    g_free (track->ipod_path);
     g_free (track->album);
+    g_free (track->artist);
     g_free (track->genre);
-    g_free (track->composer);
-    g_free (track->comment);
     g_free (track->filetype);
-    g_free (track->grouping);
+    g_free (track->comment);
     g_free (track->category);
+    g_free (track->composer);
+    g_free (track->grouping);
     g_free (track->description);
     g_free (track->podcasturl);
     g_free (track->podcastrss);
@@ -265,12 +265,20 @@ void itdb_track_free (Itdb_Track *track)
     g_free (track->tvnetwork);
     g_free (track->albumartist);
     g_free (track->keywords);
-    g_free (track->ipod_path);
+    g_free (track->sort_artist);
+    g_free (track->sort_title);
+    g_free (track->sort_album);
+    g_free (track->sort_albumartist);
+    g_free (track->sort_composer);
+    g_free (track->sort_tvshow);
+
     g_free (track->chapterdata_raw);
-    itdb_artwork_remove_thumbnails (track->artwork);
-    g_free (track->artwork);
+
+    itdb_artwork_free (track->artwork);
+
     if (track->userdata && track->userdata_destroy)
 	(*track->userdata_destroy) (track->userdata);
+
     g_free (track);
 }
 
@@ -337,14 +345,14 @@ Itdb_Track *itdb_track_duplicate (Itdb_Track *tr)
 
     /* copy strings */
     tr_dup->title = g_strdup (tr->title);
-    tr_dup->artist = g_strdup (tr->artist);
     tr_dup->album = g_strdup (tr->album);
+    tr_dup->artist = g_strdup (tr->artist);
     tr_dup->genre = g_strdup (tr->genre);
-    tr_dup->composer = g_strdup (tr->composer);
-    tr_dup->comment = g_strdup (tr->comment);
     tr_dup->filetype = g_strdup (tr->filetype);
-    tr_dup->grouping = g_strdup (tr->grouping);
+    tr_dup->comment = g_strdup (tr->comment);
     tr_dup->category = g_strdup (tr->category);
+    tr_dup->composer = g_strdup (tr->composer);
+    tr_dup->grouping = g_strdup (tr->grouping);
     tr_dup->description = g_strdup (tr->description);
     tr_dup->podcasturl = g_strdup (tr->podcasturl);
     tr_dup->podcastrss = g_strdup (tr->podcastrss);
@@ -355,6 +363,13 @@ Itdb_Track *itdb_track_duplicate (Itdb_Track *tr)
     tr_dup->albumartist = g_strdup (tr->albumartist);
     tr_dup->keywords = g_strdup (tr->keywords);
     tr_dup->ipod_path = g_strdup (tr->ipod_path);
+    tr_dup->sort_artist = g_strdup (tr->sort_artist);
+    tr_dup->sort_title = g_strdup (tr->sort_title);
+    tr_dup->sort_album = g_strdup (tr->sort_album);
+    tr_dup->sort_albumartist = g_strdup (tr->sort_albumartist);
+    tr_dup->sort_composer = g_strdup (tr->sort_composer);
+    tr_dup->sort_tvshow = g_strdup (tr->sort_tvshow);
+
 
     /* Copy chapterdata */
     if (tr->chapterdata_raw)
@@ -381,38 +396,68 @@ static gboolean itdb_track_set_thumbnails_internal (Itdb_Track *track,
 						    const gchar *filename,
 						    const guchar *image_data,
 						    gsize image_data_len,
+                                                    gpointer *pixbuf,
 						    gint rotation,
 						    GError **error)
 {					     
+    /* FIXME: this looks like it would work, but the problem is that
+     * gtkpod calls this function mostly with tracks that are not yet
+     * part of an iTunesDB. This means only the SMALL and LARGE thumbs
+     * are being added.
+     *
+     * One solution would be to add all kinds of cover thumbs and weed
+     * out the ones not supported when the ArtworkDB is written.
+     *
+     * Suggestions welcome!
+     */
+
     gboolean result = FALSE;
+    ItdbThumbType thumbtypes[] =
+	{ ITDB_THUMB_COVER_SMALL,
+	  ITDB_THUMB_COVER_LARGE,
+	  ITDB_THUMB_COVER_XLARGE,
+	  ITDB_THUMB_COVER_MEDIUM,
+	  ITDB_THUMB_COVER_SMEDIUM,
+	  ITDB_THUMB_COVER_XSMALL,
+	  -1 };
+    ItdbThumbType *thumbtype;
+    const Itdb_ArtworkFormat *formats=NULL;
 
     g_return_val_if_fail (track, FALSE);
+    g_return_val_if_fail (filename || image_data || pixbuf, FALSE);
+
+    if (track->itdb && track->itdb->device)
+    {
+	formats = itdb_device_get_artwork_formats (track->itdb->device);
+    }
 
     itdb_artwork_remove_thumbnails (track->artwork);
 
-    if (filename)
+    for (thumbtype=thumbtypes; *thumbtype!=-1; ++thumbtype)
     {
-	result = itdb_artwork_add_thumbnail (track->artwork,
-					     ITDB_THUMB_COVER_SMALL,
-					     filename, rotation, error);
-	if (result == TRUE)
+	if (filename)
+	{
 	    result = itdb_artwork_add_thumbnail (track->artwork,
-						 ITDB_THUMB_COVER_LARGE,
+						 *thumbtype,
 						 filename, rotation, error);
-    }
-    if (image_data)
-    {
-	result = itdb_artwork_add_thumbnail_from_data (track->artwork,
-						       ITDB_THUMB_COVER_SMALL,
-						       image_data,
-						       image_data_len,
-						       rotation, error);
-	if (result == TRUE)
+	}
+	if (image_data)
+	{
 	    result = itdb_artwork_add_thumbnail_from_data (track->artwork,
-							   ITDB_THUMB_COVER_LARGE,
+							   *thumbtype,
 							   image_data,
 							   image_data_len,
 							   rotation, error);
+	}
+	if (pixbuf)
+	{
+	    result = itdb_artwork_add_thumbnail_from_pixbuf (track->artwork,
+							     *thumbtype,
+							     pixbuf, rotation,
+							     error);
+        }
+	if (result == FALSE)
+	    break;  /* for (thumbtype=...) */
     }
 
     if (result == TRUE)
@@ -457,7 +502,7 @@ gboolean itdb_track_set_thumbnails (Itdb_Track *track,
     g_return_val_if_fail (track, FALSE);
     g_return_val_if_fail (filename, FALSE);
 
-    return itdb_track_set_thumbnails_internal (track, filename, NULL, 0,
+    return itdb_track_set_thumbnails_internal (track, filename, NULL, 0, NULL,
 					       0, NULL);
 }
 
@@ -487,7 +532,28 @@ gboolean itdb_track_set_thumbnails_from_data (Itdb_Track *track,
 
     return itdb_track_set_thumbnails_internal (track, NULL,
 					       image_data, image_data_len,
-					       0, NULL);
+                                               NULL, 0, NULL);
+}
+
+/**
+ * itdb_track_set_thumbnails_from_pixbuf:
+ * @track: an #Itdb_Track
+ * @pixbuf: a #GdkPixbuf used to generate the thumbnail
+ *
+ * Uses @pixbuf to generate iPod thumbnails. To save memory, the thumbnails
+ * will only be generated when necessary, ie when itdb_save() or a
+ * similar function is called.
+ *
+ * Return value: TRUE if the thumbnail could be added, FALSE otherwise.
+ **/
+gboolean itdb_track_set_thumbnails_from_pixbuf (Itdb_Track *track,
+                                                gpointer pixbuf)
+{
+    g_return_val_if_fail (track, FALSE);
+    g_return_val_if_fail (pixbuf, FALSE);
+
+    return itdb_track_set_thumbnails_internal (track, NULL, NULL, 0,
+                                               pixbuf, 0, NULL);
 }
 
 
