@@ -1,4 +1,4 @@
-/*  Time-stamp: <2007-01-09 22:00:03 jcs>
+/*  Time-stamp: <2007-11-04 09:46:51 jcs>
  *
  *  Copyright (C) 2005 Christophe Fergeau
  *
@@ -23,7 +23,7 @@
  * 
  *  This product is not supported/written/published by Apple!
  *
- *  $Id: ithumb-writer.c,v 1.26 2007/01/09 13:31:13 jcsjcs Exp $
+ *  $Id: ithumb-writer.c 1753 2007-11-04 00:48:12Z jcsjcs $
  */
 
 #include <config.h>
@@ -45,8 +45,11 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include <glib/gstdio.h>
 
-#define ITHUMB_MAX_SIZE (500L*1000L*1000L)
+/* Maximum size for .ithmb files. Reduced from 500 MB to 256 MB after
+   reports of slow iPod interface behavior */
+#define ITHUMB_MAX_SIZE (256L*1000L*1000L)
 /* for testing: */
 /*#define ITHUMB_MAX_SIZE (1L*1000L*1000L)*/
 
@@ -65,7 +68,8 @@ typedef struct _iThumbWriter iThumbWriter;
 
 static guint16 *
 pack_RGB_565 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
-	      gint horizontal_padding, gint vertical_padding)
+	      gint horizontal_padding, gint vertical_padding,
+	      guint32 *thumb_size)
 {
 	guchar *pixels;
 	guint16 *result;
@@ -86,7 +90,8 @@ pack_RGB_565 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
 	 * hardcoded in libipoddevice code, so dst_width * dst_height * 2 can't
 	 * overflow, even on an iPod containing malicious data
 	 */
-	result = g_malloc0 (img_info->width * img_info->height * 2);
+	*thumb_size = img_info->width * img_info->height * 2;
+	result = g_malloc0 (*thumb_size);
 
 	byte_order = itdb_thumb_get_byteorder (img_info->format);
 
@@ -101,12 +106,12 @@ pack_RGB_565 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
 			g = pixels[h*row_stride + w*channels + 1]; 
 			b = pixels[h*row_stride + w*channels + 2]; 
 
-			r >>= (8 - RED_BITS);
-			g >>= (8 - GREEN_BITS);
-			b >>= (8 - BLUE_BITS);
-			r = (r << RED_SHIFT) & RED_MASK;
-			g = (g << GREEN_SHIFT) & GREEN_MASK;
-			b = (b << BLUE_SHIFT) & BLUE_MASK;
+			r >>= (8 - RED_BITS_565);
+			g >>= (8 - GREEN_BITS_565);
+			b >>= (8 - BLUE_BITS_565);
+			r = (r << RED_SHIFT_565) & RED_MASK_565;
+			g = (g << GREEN_SHIFT_565) & GREEN_MASK_565;
+			b = (b << BLUE_SHIFT_565) & BLUE_MASK_565;
 			result[line + w + horizontal_padding] =
 			    get_gint16 (r | g | b, byte_order);
 		}
@@ -114,33 +119,288 @@ pack_RGB_565 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
 	return result;
 }
 
+static guint16 *
+pack_RGB_555 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
+	      gint horizontal_padding, gint vertical_padding,
+	      guint32 *thumb_size)
+{
+	guchar *pixels;
+	guint16 *result;
+	gint row_stride;
+	gint channels;
+	gint width;
+	gint height;
+	gint w;
+	gint h;
+	gint byte_order;
+
+	g_object_get (G_OBJECT (pixbuf), 
+		      "rowstride", &row_stride, "n-channels", &channels,
+		      "height", &height, "width", &width,
+		      "pixels", &pixels, NULL);
+	g_return_val_if_fail ((width <= img_info->width) && (height <= img_info->height), NULL);
+	/* dst_width and dst_height come from a width/height database 
+	 * hardcoded in libipoddevice code, so dst_width * dst_height * 2 can't
+	 * overflow, even on an iPod containing malicious data
+	 */
+
+	*thumb_size = img_info->width * img_info->height * 2;
+	result = g_malloc0 (*thumb_size);
+
+	byte_order = itdb_thumb_get_byteorder (img_info->format);
+
+	for (h = 0; h < height; h++) {
+	        gint line = (h+vertical_padding)*img_info->width;
+		for (w = 0; w < width; w++) {
+			gint r;
+			gint g;
+			gint b;
+			gint a;
+
+			r = pixels[h*row_stride + w*channels];
+			g = pixels[h*row_stride + w*channels + 1]; 
+			b = pixels[h*row_stride + w*channels + 2]; 
+
+			r >>= (8 - RED_BITS_555);
+			g >>= (8 - GREEN_BITS_555);
+			b >>= (8 - BLUE_BITS_555);
+			a = (1 << ALPHA_SHIFT_555) & ALPHA_MASK_555;
+			r = (r << RED_SHIFT_555) & RED_MASK_555;
+			g = (g << GREEN_SHIFT_555) & GREEN_MASK_555;
+			b = (b << BLUE_SHIFT_555) & BLUE_MASK_555;
+			result[line + w + horizontal_padding] =
+			    get_gint16 (a | r | g | b, byte_order);
+			/* I'm not sure if the highest bit really is
+			   the alpha channel. For now I'm just setting
+			   this bit because that's what I have seen. */
+		}
+	}
+	return result;
+}
+
+
+static guint16 *
+pack_RGB_888 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
+	      gint horizontal_padding, gint vertical_padding,
+	      guint32 *thumb_size)
+{
+	guchar *pixels;
+	guint32 *result;
+	gint row_stride;
+	gint channels;
+	gint width;
+	gint height;
+	gint w;
+	gint h;
+	gint byte_order;
+
+	g_object_get (G_OBJECT (pixbuf), 
+		      "rowstride", &row_stride, "n-channels", &channels,
+		      "height", &height, "width", &width,
+		      "pixels", &pixels, NULL);
+	g_return_val_if_fail ((width <= img_info->width) && (height <= img_info->height), NULL);
+	/* dst_width and dst_height come from a width/height database 
+	 * hardcoded in libipoddevice code, so dst_width * dst_height * 2 can't
+	 * overflow, even on an iPod containing malicious data
+	 */
+	*thumb_size = img_info->width * img_info->height * 4;
+	result = g_malloc0 (*thumb_size);
+
+	byte_order = itdb_thumb_get_byteorder (img_info->format);
+
+	for (h = 0; h < height; h++) {
+	        gint line = (h+vertical_padding)*img_info->width;
+		for (w = 0; w < width; w++) {
+			guint32 r;
+			guint32 g;
+			guint32 b;
+			guint32 a;
+
+			r = pixels[h*row_stride + w*channels];
+			g = pixels[h*row_stride + w*channels + 1]; 
+			b = pixels[h*row_stride + w*channels + 2]; 
+
+			r >>= (8 - RED_BITS_888);
+			g >>= (8 - GREEN_BITS_888);
+			b >>= (8 - BLUE_BITS_888);
+			a = (0xff << ALPHA_SHIFT_888) & ALPHA_MASK_888;
+			r = (r << RED_SHIFT_888) & RED_MASK_888;
+			g = (g << GREEN_SHIFT_888) & GREEN_MASK_888;
+			b = (b << BLUE_SHIFT_888) & BLUE_MASK_888;
+			result[line + w + horizontal_padding] =
+			    get_gint32 (a | r | g | b, byte_order);
+			/* I'm not sure if the highest bit really is
+			   the alpha channel. For now I'm just setting
+			   these bits because that's what I have seen. */
+		}
+	}
+	return (guint16 *)result;
+}
+
+
+static guint16 *derange_pixels (guint16 *pixels_s, guint16 *pixels_d,
+				gint width, gint height, gint row_stride)
+{
+    g_return_val_if_fail (width == height, pixels_s);
+
+    if (pixels_s == NULL)
+    {
+	pixels_s = g_malloc0 (sizeof (guint16)*width*height);
+    }
+
+    if (width == 1)
+    {
+	*pixels_s = *pixels_d;
+    }
+    else
+    {
+	derange_pixels (pixels_s + 0,
+			pixels_d + 0 + 0,
+			width/2, height/2,
+			row_stride);
+	derange_pixels (pixels_s + (width/2)*(height/2),
+			pixels_d + (height/2)*row_stride + 0,
+			width/2, height/2,
+			row_stride);
+	derange_pixels (pixels_s + 2*(width/2)*(height/2),
+			pixels_d + width/2,
+			width/2, height/2,
+			row_stride);
+	derange_pixels (pixels_s + 3*(width/2)*(height/2),
+			pixels_d + (height/2)*row_stride + width/2,
+			width/2, height/2,
+			row_stride);
+    }
+    
+    return pixels_s;
+}
+
+static guint16 *
+pack_rec_RGB_555 (GdkPixbuf *pixbuf, const Itdb_ArtworkFormat *img_info,
+		  gint horizontal_padding, gint vertical_padding,
+		  guint32 *thumb_size)
+{
+    guint16 *pixels;
+    guint16 *deranged_pixels = NULL;
+
+    pixels = pack_RGB_555 (pixbuf, img_info,
+			   horizontal_padding, vertical_padding,
+			   thumb_size);
+
+    if (pixels)
+    {
+	deranged_pixels = derange_pixels (NULL, pixels,
+					  img_info->width, img_info->height,
+					  img_info->width);
+	g_free (pixels);
+    }
+
+    return deranged_pixels;
+}
 
-/* pack_UYVY() is adapted from imgconvert.c from the GPixPod project
- * (www.gpixpod.org) */
 static guchar *
-pack_UYVY (GdkPixbuf *orig_pixbuf, const Itdb_ArtworkFormat *img_info,
-	   gint horizontal_padding, gint vertical_padding)
+pack_I420 (GdkPixbuf *orig_pixbuf, const Itdb_ArtworkFormat *img_info,
+	   gint horizontal_padding, gint vertical_padding,
+	   guint32 *thumb_size)
 {
     GdkPixbuf *pixbuf;
-    guchar *pixels, *yuvdata;
-    gint width;
-    gint height;
+    gint width, height;
     gint orig_height, orig_width;
-    gint x = 0;
-    gint z = 0;
-    gint z2 = 0;
-    gint h = 0;
-    gint r0, g0, b0, r1, g1, b1, r2, g2, b2, r3, g3, b3;
     gint rowstride;
+    gint h, z;
+    guchar *pixels, *yuvdata;
     gint yuvsize, halfyuv;
-    gint alphabit, rgbpx;
-    gint exc;
+    gint ustart, vstart;
 
     g_return_val_if_fail (img_info, NULL);
 
     width = img_info->width;
     height = img_info->height;
 
+    g_object_get (G_OBJECT (orig_pixbuf), 
+		  "height", &orig_height, "width", &orig_width, NULL);
+
+    /* copy into new pixmap with padding applied */
+    pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+			     gdk_pixbuf_get_has_alpha (orig_pixbuf),
+			     8,
+			     width, height);
+    gdk_pixbuf_copy_area (orig_pixbuf, 0, 0, orig_width, orig_height,
+			  pixbuf, horizontal_padding, vertical_padding);
+
+    g_object_get (G_OBJECT (pixbuf), 
+		  "rowstride", &rowstride,
+		  "pixels", &pixels, NULL);
+
+    halfyuv = width*height;
+
+    yuvsize = 2*halfyuv;
+    *thumb_size = yuvsize;
+
+    yuvdata = g_malloc (yuvsize);
+
+    ustart = halfyuv;
+    vstart = ustart + halfyuv/4;
+
+    /* FIXME: consider rowstride -- currently we assume rowstride==width */
+    for (z=0,h=0; h < halfyuv; ++h)
+    {
+	gint r,g,b;
+	gint u, v, y;
+	gint row, col;
+
+	r = pixels[z];
+	g = pixels[z+1];
+	b = pixels[z+2];
+
+	y = (( 66*r + 129*g +  25*b + 128) >> 8) + 16;
+	u = ((-38*r -  74*g + 112*b + 128) >> 8) + 128;
+	v = ((112*r -  94*g -  18*b + 128) >> 8) + 128;
+
+	row = h / width;
+	col = h % width;
+
+	yuvdata[h] = y;
+	yuvdata[ustart + (row/2)*(width/2) + col/2] = u;
+	yuvdata[vstart + (row/2)*(width/2) + col/2] = v;
+
+	if (gdk_pixbuf_get_has_alpha(pixbuf))
+	    z+=4;
+	else
+	    z+=3;
+    }
+
+    return yuvdata;
+}
+
+/* pack_UYVY() is adapted from imgconvert.c from the GPixPod project
+ * (www.gpixpod.org) */
+static guchar *
+pack_UYVY (GdkPixbuf *orig_pixbuf, const Itdb_ArtworkFormat *img_info,
+	   gint horizontal_padding, gint vertical_padding,
+	   guint32 *thumb_size)
+{
+    GdkPixbuf *pixbuf;
+    guchar *pixels, *yuvdata;
+    gint width;
+    gint height;
+    gint orig_height, orig_width;
+    gint x = 0;
+    gint z = 0;
+    gint z2 = 0;
+    gint h = 0;
+    gint r0, g0, b0, r1, g1, b1, r2, g2, b2, r3, g3, b3;
+    gint rowstride;
+    gint yuvsize, halfyuv;
+    gint alphabit, rgbpx;
+    gint exc;
+
+    g_return_val_if_fail (img_info, NULL);
+
+    width = img_info->width;
+    height = img_info->height;
+    *thumb_size = 2*width*height;
+
     g_object_get (G_OBJECT (orig_pixbuf), 
 		  "height", &orig_height, "width", &orig_width, NULL);
 
@@ -157,6 +417,7 @@ pack_UYVY (GdkPixbuf *orig_pixbuf, const Itdb_ArtworkFormat *img_info,
 		  "pixels", &pixels, NULL);
 
     yuvsize = width*2*height;
+
     yuvdata = g_malloc (yuvsize);
     halfyuv = yuvsize/2;
     if (gdk_pixbuf_get_has_alpha(pixbuf))
@@ -289,7 +550,7 @@ ipod_image_get_ithmb_filename (const char *mount_point, gint correlation_id, gin
 /*	printf ("%s %s\n", buf, filename);*/
 
 	g_free (buf);
-
+        g_free (artwork_dir);
 	return filename;
 }
 
@@ -355,6 +616,14 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
 	thumb->image_data = NULL;
 	thumb->image_data_len = 0;
     }
+    else if (thumb->pixbuf)
+    {
+        pixbuf = gdk_pixbuf_scale_simple (GDK_PIXBUF(thumb->pixbuf),
+                                          width, height,
+                                          GDK_INTERP_BILINEAR);
+        g_object_unref (thumb->pixbuf);
+        thumb->pixbuf = NULL;
+    }
 
     if (pixbuf == NULL)
     {
@@ -414,6 +683,10 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
 	break;
     case ITDB_THUMB_COVER_LARGE:
     case ITDB_THUMB_COVER_SMALL:
+    case ITDB_THUMB_COVER_XLARGE:
+    case ITDB_THUMB_COVER_MEDIUM:
+    case ITDB_THUMB_COVER_SMEDIUM:
+    case ITDB_THUMB_COVER_XSMALL:
 	thumb->filename = g_strdup_printf (":F%d_%d.ithmb", 
 					   writer->img_info->correlation_id,
 					   writer->current_file_index);
@@ -443,7 +716,6 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
     thumb->width = thumb->horizontal_padding + width;
     thumb->height = thumb->vertical_padding + height;
     thumb->offset = writer->cur_offset;
-    thumb->size = writer->img_info->width * writer->img_info->height * 2;
 
     switch (writer->img_info->format)
     {
@@ -456,12 +728,61 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
     case THUMB_FORMAT_RGB565_BE:
 	pixels = pack_RGB_565 (pixbuf, writer->img_info,
 			       thumb->horizontal_padding,
-			       thumb->vertical_padding);
+			       thumb->vertical_padding,
+			       &thumb->size);
 	break;
-    case THUMB_FORMAT_UYVY:
+    case THUMB_FORMAT_RGB555_LE_90:
+    case THUMB_FORMAT_RGB555_BE_90:
+	/* FIXME: actually the previous two might require
+	   different treatment (used on iPod Photo for the full
+	   screen photo thumbnail) */
+    case THUMB_FORMAT_RGB555_LE:
+    case THUMB_FORMAT_RGB555_BE:
+	pixels = pack_RGB_555 (pixbuf, writer->img_info,
+			       thumb->horizontal_padding,
+			       thumb->vertical_padding,
+			       &thumb->size);
+	break;
+    case THUMB_FORMAT_RGB888_LE_90:
+    case THUMB_FORMAT_RGB888_BE_90:
+	/* FIXME: actually the previous two might require
+	   different treatment (used on iPod Photo for the full
+	   screen photo thumbnail) */
+    case THUMB_FORMAT_RGB888_LE:
+    case THUMB_FORMAT_RGB888_BE:
+	pixels = pack_RGB_888 (pixbuf, writer->img_info,
+			       thumb->horizontal_padding,
+			       thumb->vertical_padding,
+			       &thumb->size);
+	break;
+    case THUMB_FORMAT_REC_RGB555_LE_90:
+    case THUMB_FORMAT_REC_RGB555_BE_90:
+	/* FIXME: actually the previous two might require
+	   different treatment (used on iPod Photo for the full
+	   screen photo thumbnail) */
+    case THUMB_FORMAT_REC_RGB555_LE:
+    case THUMB_FORMAT_REC_RGB555_BE:
+	pixels = pack_rec_RGB_555 (pixbuf, writer->img_info,
+				   thumb->horizontal_padding,
+				   thumb->vertical_padding,
+				   &thumb->size);
+	break;
+    case THUMB_FORMAT_EXPERIMENTAL_LE:
+    case THUMB_FORMAT_EXPERIMENTAL_BE:
+	break;
+    case THUMB_FORMAT_UYVY_BE:
+    case THUMB_FORMAT_UYVY_LE:
 	pixels = pack_UYVY (pixbuf, writer->img_info,
 			    thumb->horizontal_padding,
-			    thumb->vertical_padding);
+			    thumb->vertical_padding,
+	                    &thumb->size);
+	break;
+    case THUMB_FORMAT_I420_BE:
+    case THUMB_FORMAT_I420_LE:
+	pixels = pack_I420 (pixbuf, writer->img_info,
+			    thumb->horizontal_padding,
+			    thumb->vertical_padding,
+	                    &thumb->size);
 	break;
     }
 
@@ -472,6 +793,7 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
     {
 	return FALSE;
     }
+
     if (fwrite (pixels, thumb->size, 1, writer->f) != 1) {
 	g_free (pixels);
 	g_print ("Error writing to file: %s\n", strerror (errno));
@@ -480,6 +802,23 @@ ithumb_writer_write_thumbnail (iThumbWriter *writer,
     g_free (pixels);
     writer->cur_offset += thumb->size;
 
+    if (writer->img_info->padding != 0)
+    {
+	gint padding = writer->img_info->padding - thumb->size;
+	g_return_val_if_fail (padding >= 0, TRUE);
+	if (padding != 0)
+	{
+            /* FIXME: check if a simple fseek() will do the same */
+	    gchar *pad_bytes = g_malloc0 (padding);
+	    if (fwrite (pad_bytes, padding, 1, writer->f) != 1) {
+		g_free (pad_bytes);
+		g_print ("Error writing to file: %s\n", strerror (errno));
+		return FALSE;
+	    }
+	    g_free (pad_bytes);
+	    writer->cur_offset += padding;
+	}
+    }
     return TRUE;
 }
 
@@ -938,22 +1277,16 @@ itdb_write_ithumb_files (Itdb_DB *db)
 	while (format->type != -1) {
 		iThumbWriter *writer;
 
-		switch (format->type) {
-		case ITDB_THUMB_COVER_SMALL:
-		case ITDB_THUMB_COVER_LARGE:
-		case ITDB_THUMB_PHOTO_SMALL:
-		case ITDB_THUMB_PHOTO_LARGE:
-		case ITDB_THUMB_PHOTO_FULL_SCREEN:
-		case ITDB_THUMB_PHOTO_TV_SCREEN:
-		        ithmb_rearrange_existing_thumbnails (db, format );
-			writer = ithumb_writer_new (mount_point, 
-						    format,
-						    db->db_type, 
-						    device->byte_order);
-			if (writer != NULL) {
-				writers = g_list_prepend (writers, writer);
-			}
-			break;
+		if (itdb_thumb_type_is_valid_for_db (format->type, db->db_type))
+		{
+		    ithmb_rearrange_existing_thumbnails (db, format );
+		    writer = ithumb_writer_new (mount_point, 
+						format,
+						db->db_type, 
+						device->byte_order);
+		    if (writer != NULL) {
+			writers = g_list_prepend (writers, writer);
+		    }
 		}
 		format++;
 	}
